@@ -46,7 +46,7 @@
 * revision history - 
 *          David Thompson  - Added edge/face blocks/sets
 *
-*  $Id: expinix.c 2928 2008-07-11 17:45:07Z friedmud $
+*  $Id: expinix.c 4347 2011-04-14 14:08:04Z jwpeterson $
 *
 *****************************************************************************/
 
@@ -516,6 +516,9 @@ int ex_put_init_ext (int   exoid,
     zero_id_status(exoid, VAR_FS_STAT,  VAR_FS_IDS,  model->num_face_sets, zeros);
     zero_id_status(exoid, VAR_SS_STAT,  VAR_SS_IDS,  model->num_side_sets, zeros);
     zero_id_status(exoid, VAR_ELS_STAT, VAR_ELS_IDS, model->num_elem_sets, zeros);
+    
+    /* clean up after ourselves */
+    free(zeros);
   }  
   return (EX_NOERR);
   
