@@ -1,4 +1,4 @@
-// $Id: dof_map.h 3504 2009-10-21 18:58:45Z knezed01 $
+// $Id: dof_map.h 4360 2011-04-19 07:23:18Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -34,21 +34,26 @@
 #include "enum_order.h"
 #include "reference_counted_object.h"
 #include "libmesh.h" // libMesh::invalid_uint
-#include "vector_value.h" // RealVectorValue
-#include "system.h" // System::Variable
+#include "variable.h"
 #include "threads.h"
 #include "threads_allocators.h"
 #include "elem_range.h"
 
+namespace libMesh
+{
 
 // Forward Declarations
+class CouplingMatrix;
 class DofMap;
 class DofObject;
 class Elem;
+class FEType;
 class MeshBase;
 class Mesh;
-class FEType;
-class CouplingMatrix;
+class PeriodicBoundary;
+class PeriodicBoundaries;
+namespace SparsityPattern { class Build; }
+class System;
 template <typename T> class DenseVectorBase;
 template <typename T> class DenseVector;
 template <typename T> class DenseMatrix;
@@ -57,100 +62,6 @@ template <typename T> class NumericVector;
 
 
 
-
-// ------------------------------------------------------------
-// Sparsity Pattern
-
-/**
- * This defines the sparsity pattern, or graph, of a sparse matrix.
- * The format is quite simple -- the global indices of the nonzero entries
- * in each row are packed into a vector.  The global indices (i,j) of the 
- * nth nonzero entry of row i are given by j = sparsity_pattern[i][n];
- */
-namespace SparsityPattern // use a namespace so member classes can be forward-declared.
-{
-  typedef std::vector<unsigned int, Threads::scalable_allocator<unsigned int> > Row;
-  class Graph : public std::vector<Row> {};
-  
-  /**
-   * Splices the two sorted ranges [begin,middle) and [middle,end)
-   * into one sorted range [begin,end).  This method is much like
-   * std::inplace_merge except it assumes the intersection
-   * of the two sorted ranges is empty and that any element in
-   * each range occurs only once in that range.  Additionally,
-   * this sort occurs in-place, while std::inplace_merge may
-   * use a temporary buffer.
-   */ 
-  template<typename BidirectionalIterator>
-  static void sort_row (const BidirectionalIterator begin,
-			BidirectionalIterator       middle,
-			const BidirectionalIterator end);
-
-  /**
-   * This helper class can be called on multiple threads to compute 
-   * the sparsity pattern (or graph) of the sparse matrix resulting
-   * from the discretization.  This pattern may be used directly by
-   * a particular sparse matrix format (e.g. \p LaspackMatrix)
-   * or indirectly (e.g. \p PetscMatrix).  In the latter case the
-   * number of nonzeros per row of the matrix is needed for efficient 
-   * preallocation.  In this case it suffices to provide estimate
-   * (but bounding) values, and in this case the threaded method can
-   * take some short-cuts for efficiency.
-   */
-  class Build
-  {
-  private:
-    const MeshBase &mesh;
-    const DofMap &dof_map;
-    const CouplingMatrix *dof_coupling;
-    const bool implicit_neighbor_dofs;
-    const bool need_full_sparsity_pattern;
-
-  public:
-
-    SparsityPattern::Graph sparsity_pattern;
-    std::vector<unsigned int> n_nz;
-    std::vector<unsigned int> n_oz;
-    
-    Build (const MeshBase &mesh_in,
-	   const DofMap &dof_map_in,
-	   const CouplingMatrix *dof_coupling_in,
-	   const bool implicit_neighbor_dofs_in,
-	   const bool need_full_sparsity_pattern_in) :
-      mesh(mesh_in),
-      dof_map(dof_map_in),
-      dof_coupling(dof_coupling_in),
-      implicit_neighbor_dofs(implicit_neighbor_dofs_in),
-      need_full_sparsity_pattern(need_full_sparsity_pattern_in)
-    {}
-
-    Build (Build &other, Threads::split) :
-      mesh(other.mesh),
-      dof_map(other.dof_map),
-      dof_coupling(other.dof_coupling),
-      implicit_neighbor_dofs(other.implicit_neighbor_dofs),
-      need_full_sparsity_pattern(other.need_full_sparsity_pattern)
-    {}
-
-    void operator()(const ConstElemRange &range);
-
-    void join (const Build &other);
-  };
-
-#if defined(__GNUC__) && (__GNUC__ < 4) && !defined(__INTEL_COMPILER)
-  /**
-   * Dummy function that does nothing but can be used to prohibit
-   * compiler optimization in some situations where some compilers
-   * have optimization bugs.
-   */
-  void _dummy_function(void);
-#endif
-  
-}
-
-
-
-
 // ------------------------------------------------------------
 // AMR constraint matrix types
 
@@ -177,54 +88,6 @@ class DofConstraints : public std::map<unsigned int,
 #endif // LIBMESH_ENABLE_AMR || LIBMESH_ENABLE_PERIODIC
 
   
-// ------------------------------------------------------------
-// Periodic boundary conditions information
-
-#ifdef LIBMESH_ENABLE_PERIODIC
-/**
- * The definition of a periodic boundary.
- */
-class PeriodicBoundary
-{
-public:
-  // The boundary ID of this boundary and it's counterpart
-  unsigned int myboundary,
-	       pairedboundary;
-
-  // One of these days we'll support rotated boundaries
-  // RealTensor rotation_matrix;
-
-  // The vector which is added to points in myboundary
-  // to produce corresponding points in pairedboundary
-  RealVectorValue translation_vector;
-};
-
-
-/** 
- * The constraint matrix storage format. 
- * We're using a class instead of a typedef to allow forward
- * declarations and future flexibility.  Is there some issue with
- * deriving from standard containers, i.e. don't do it because they
- * don't have virtual destructors?
- */
-class PeriodicBoundaries : public std::map<unsigned int, PeriodicBoundary>
-{
-public:
-  PeriodicBoundary *boundary(unsigned int id);
-
-  PeriodicBoundaries() {}
-
-  ~PeriodicBoundaries();
-
-  // The periodic neighbor of \p e in direction \p side, if it
-  // exists.  NULL otherwise
-  const Elem *neighbor(unsigned int boundary_id, const MeshBase &mesh, const Elem *e, unsigned int side);
-
-private:
-};
-#endif // LIBMESH_ENABLE_PERIODIC
-
-
   
 // ------------------------------------------------------------
 // DofMap class definition
@@ -274,6 +137,13 @@ public:
    */
   void compute_sparsity (const MeshBase&);
 
+  /**
+   * Attach a function pointer to use as a callback to populate the
+   * send_list with extra entries.
+   */
+  void attach_extra_send_list_function(void (*func)(std::vector<unsigned int> &, void *), void * context = NULL)
+    { _extra_send_list_function = func; _extra_send_list_context = context; }
+
   /**
    * Takes the \p _send_list vector (which may have duplicate entries)
    * and sorts it.  The duplicate entries are then removed, resulting in
@@ -310,12 +180,12 @@ public:
    * Add an unknown of order \p order and finite element type
    * \p type to the system of equations.
    */
-  void add_variable (const System::Variable &var);
+  void add_variable (const Variable &var);
   
   /**
    * @returns the variable description object for variable \p c.
    */
-  const System::Variable& variable (const unsigned int c) const;
+  const Variable& variable (const unsigned int c) const;
 
   /**
    * @returns the approximation order for variable \p c.
@@ -362,6 +232,14 @@ public:
    */
   unsigned int first_dof(const unsigned int proc = libMesh::processor_id()) const
   { libmesh_assert(proc < _first_df.size()); return _first_df[proc]; }
+
+#ifdef LIBMESH_ENABLE_AMR
+  /**
+   * Returns the first old dof index that is local to subdomain \p proc.
+   */
+  unsigned int first_old_dof(const unsigned int proc = libMesh::processor_id()) const
+  { libmesh_assert(proc < _first_old_df.size()); return _first_old_df[proc]; }
+#endif //LIBMESH_ENABLE_AMR
   
   /**
    * Returns the last dof index that is local to subdomain \p proc.
@@ -377,6 +255,15 @@ public:
    */
   unsigned int end_dof(const unsigned int proc = libMesh::processor_id()) const
   { libmesh_assert(proc < _end_df.size()); return _end_df[proc]; }
+
+#ifdef LIBMESH_ENABLE_AMR
+  /**
+   * Returns the first old dof index that is after all indices local to subdomain \p proc.
+   * Analogous to the end() member function of STL containers.
+   */
+  unsigned int end_old_dof(const unsigned int proc = libMesh::processor_id()) const
+  { libmesh_assert(proc < _end_old_df.size()); return _end_old_df[proc]; }
+#endif //LIBMESH_ENABLE_AMR
   
   /**
    * Returns the first local degree of freedom index for variable \p var.
@@ -418,6 +305,14 @@ public:
 		           const unsigned int vn,
 		           const bool old_dofs=false) const;
 
+  /**
+   * Returns \p true iff all degree of freedom indices in
+   * \p dof_indices are either local indices or in the \p send_list.
+   * Note that this is an O(logN) operation, not O(1); we don't cache
+   * enough information for O(1) right now.
+   */
+  bool all_semilocal_indices (const std::vector<unsigned int>& dof_indices) const;
+
   /**
    * Tells other library functions whether or not this problem
    * includes coupling between dofs in neighboring cells, as can
@@ -477,6 +372,18 @@ public:
   void add_constraint_row (const unsigned int dof_number,
 			   const DofConstraintRow& constraint_row,
 			   const bool forbid_constraint_overwrite = true);
+
+  /**
+   * Returns an iterator pointing to the first constraint row
+   */
+  DofConstraints::const_iterator constraint_rows_begin() const
+    { return _dof_constraints.begin(); }
+  
+  /**
+   * Returns an iterator pointing just past the last constraint row
+   */
+  DofConstraints::const_iterator constraint_rows_end() const
+    { return _dof_constraints.end(); }
   
   /**
    * @returns true if the degree of freedom dof is constrained,
@@ -485,9 +392,9 @@ public:
   bool is_constrained_dof (const unsigned int dof) const;
   
   /**
-   * Prints the \p _dof_constraints data structure.
+   * Prints the whole \p _dof_constraints data structure.
    */
-  void print_dof_constraints(std::ostream& os=std::cout) const;
+  void print_dof_constraints(std::ostream& os=libMesh::out) const;
 
   /**
    * Tests the constrained degrees of freedom on the numeric vector \p v, which
@@ -600,12 +507,25 @@ public:
    */
   void add_periodic_boundary (const PeriodicBoundary& periodic_boundary);
 
+  /**
+   * Add a periodic boundary pair
+   *
+   * @param boundary - primary boundary
+   * @param inverse_boundary - inverse boundary
+   */
+  void add_periodic_boundary (PeriodicBoundary * boundary, PeriodicBoundary * inverse_boundary);
+
   /**
    * @returns true if the boundary given by \p boundaryid is periodic,
    * false otherwise
    */
   bool is_periodic_boundary (const unsigned int boundaryid) const;
 
+  PeriodicBoundaries * get_periodic_boundaries()
+    {
+      return _periodic_boundaries;
+    }
+
 #endif // LIBMESH_ENABLE_PERIODIC
 
 
@@ -661,6 +581,16 @@ public:
    */
   void clear ();
   
+  /**
+   * Prints summary info about the sparsity bandwidth and constraints.
+   */
+  void print_info(std::ostream& os=libMesh::out) const;
+
+  /**
+   * Gets summary info about the sparsity bandwidth and constraints.
+   */
+  std::string get_info() const;
+
   /**
    * Degree of freedom coupling.  If left empty each DOF
    * couples to all others.  Can be used to reduce memory
@@ -685,7 +615,7 @@ private:
   unsigned int sys_number() const;
 
   /**
-   * @invalidates all active DofObject dofs for this system
+   * Invalidates all active DofObject dofs for this system
    */
   void invalidate_dofs(MeshBase& mesh) const;
 
@@ -781,7 +711,7 @@ private:
   /**
    * The finite element type for each variable.
    */
-  std::vector<System::Variable> _variables;
+  std::vector<Variable> _variables;
 
   /**
    * The number of the system we manage DOFs for.
@@ -816,6 +746,16 @@ private:
    */
   std::vector<unsigned int> _send_list;
 
+  /**
+   * A function pointer to a function to call to add extra entries to the send list
+   */
+  void (*_extra_send_list_function)(std::vector<unsigned int> &, void *);
+
+  /**
+   * A pointer associcated with the extra send list that can optionally be passed in
+   */
+  void * _extra_send_list_context;
+
   /**
    * The number of on-processor nonzeros in my portion of the
    * global matrix.
@@ -846,6 +786,16 @@ private:
    */
   unsigned int _n_old_dfs;
 
+  /**
+   * First old DOF index on processor \p p.
+   */
+  std::vector<unsigned int> _first_old_df;
+
+  /**
+   * Last old DOF index (plus 1) on processor \p p.
+   */
+  std::vector<unsigned int> _end_old_df;
+
 #endif
 
 #if defined(LIBMESH_ENABLE_AMR) || defined(LIBMESH_ENABLE_PERIODIC)
@@ -864,30 +814,15 @@ private:
    * Data structure containing periodic boundaries.  The ith
    * entry is the constraint matrix row for boundaryid i.
    */
-  PeriodicBoundaries _periodic_boundaries;
+  PeriodicBoundaries *_periodic_boundaries;
 #endif
 
   friend class SparsityPattern::Build;
 };
 
 
-
 // ------------------------------------------------------------
 // Dof Map inline member functions
-inline
-DofMap::DofMap(const unsigned int number) :
-  _dof_coupling(NULL),
-  _sys_number(number),
-//  _matrix(NULL),
-  _n_dfs(0),
-  _n_SCALAR_dofs(0)
-#ifdef LIBMESH_ENABLE_AMR
-  , _n_old_dfs(0)
-#endif
-{
-  _matrices.clear();
-}
-
 
 #if defined(LIBMESH_ENABLE_AMR) || defined(LIBMESH_ENABLE_PERIODIC)
 
@@ -903,20 +838,6 @@ bool DofMap::is_constrained_dof (const unsigned int dof) const
 #endif
 
 
-#ifdef LIBMESH_ENABLE_PERIODIC
-
-inline
-bool DofMap::is_periodic_boundary (const unsigned int boundaryid) const
-{
-  if (_periodic_boundaries.count(boundaryid) != 0)
-    return true;
-
-  return false;
-}
-
-#endif
-
-
 inline
 unsigned int DofMap::sys_number() const
 {
@@ -924,84 +845,6 @@ unsigned int DofMap::sys_number() const
 }
 
 
-
-// ------------------------------------------------------------
-// SparsityPattern inline member functions
-template<typename BidirectionalIterator>
-inline
-void SparsityPattern::sort_row (const BidirectionalIterator begin,
-				BidirectionalIterator       middle,
-				const BidirectionalIterator end)
-{
-  if ((begin == middle) || (middle == end)) return;
-
-  libmesh_assert (std::distance (begin,  middle) > 0);
-  libmesh_assert (std::distance (middle, end)    > 0);
-  libmesh_assert (std::unique (begin,  middle) == middle);
-  libmesh_assert (std::unique (middle, end)    == end);
-  
-  while (middle != end)
-    {
-      BidirectionalIterator
-	b = middle,
-	a = b-1;
-
-      // Bubble-sort the middle value downward
-      while (!(*a < *b)) // *a & *b are less-than comparable, so use <
-	{
-	  std::swap (*a, *b);
-
-#if defined(__GNUC__) && (__GNUC__ < 4) && !defined(__INTEL_COMPILER)
-	  /* Prohibit optimization at this point since gcc 3.3.5 seems
-	     to have a bug.  */
-	  SparsityPattern::_dummy_function();
-#endif
-
-	  if (a == begin) break;
-	  
-	  b=a;
-	  --a;
-	}
-      
-      ++middle;
-    }
-
-  // Assure the algorithm worked if we are in DEBUG mode
-#ifdef DEBUG
-  {
-    // SGI STL extension!
-    // libmesh_assert (std::is_sorted(begin,end));
-    
-    BidirectionalIterator
-      prev  = begin,
-      first = begin;
-
-    for (++first; first != end; prev=first, ++first)
-      if (*first < *prev)
-	libmesh_assert(false);
-  }  
-#endif
-
-  // Make sure the two ranges did not contain any common elements
-  libmesh_assert (std::unique (begin, end) == end);
-}
-
-
-// ------------------------------------------------------------
-// PeriodicBoundary inline member functions
-#ifdef LIBMESH_ENABLE_PERIODIC
-
-inline
-PeriodicBoundary *PeriodicBoundaries::boundary(unsigned int id)
-{
-  iterator i = this->find(id);
-  if (i == this->end())
-    return NULL;
-  return &i->second;
-}
-
-#endif // LIBMESH_ENABLE_PERIODIC
-
 #if !defined(LIBMESH_ENABLE_AMR) && !defined(LIBMESH_ENABLE_PERIODIC)
   //--------------------------------------------------------------------
   // Constraint-specific methods get inlined into nothing if
@@ -1036,4 +879,6 @@ inline void DofMap::enforce_constraints_exactly (const System &,
 
 #endif // !defined(LIBMESH_ENABLE_AMR) && !defined(LIBMESH_ENABLE_PERIODIC)
 
+} // namespace libMesh
+
 #endif // __dof_map_h__
