@@ -1,4 +1,4 @@
-// $Id: id_types.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: id_types.h 3874 2010-07-02 21:57:26Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -22,6 +22,11 @@
 #ifndef __id_types_h__
 #define __id_types_h__
 
+#include <limits>
+
+namespace libMesh
+{
+
 // A useful way to debug:
 #if 0
 class TestClass {
@@ -39,4 +44,6 @@ typedef TestClass subdomain_id_type;
 
 typedef unsigned char subdomain_id_type;
 
+} // namespace libMesh
+
 #endif // end #ifndef __id_types_h__
