@@ -1,4 +1,4 @@
-// $Id: print_trace.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: print_trace.h 3874 2010-07-02 21:57:26Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -24,9 +24,14 @@
 
 #include "libmesh_config.h"
 
+namespace libMesh
+{
+
 /*
  * Print a stack trace (for code compiled with gcc)
  */
 void print_trace(std::ostream &out = std::cerr);
 
+} // namespace libMesh
+
 #endif
