@@ -1,4 +1,4 @@
-// $Id: enum_point_locator_type.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: enum_point_locator_type.h 3874 2010-07-02 21:57:26Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -24,7 +24,7 @@
 
 // ------------------------------------------------------------
 // enum PointLocatorType definition
-namespace MeshEnums {
+namespace libMeshEnums {
   
   /**
    * \enum MeshEnums::PointLocatorType defines an \p enum for the types
@@ -36,7 +36,7 @@ namespace MeshEnums {
 			 INVALID_LOCATOR};
 }
 
-using namespace MeshEnums;
+using namespace libMeshEnums;
 
 #endif // #ifndef __enum_point_locator_type_h__
 
