@@ -1,4 +1,4 @@
-// $Id: fe.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: fe.h 3874 2010-07-02 21:57:26Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -28,6 +28,9 @@
 #include "fe_base.h"
 #include "libmesh.h"
 
+namespace libMesh
+{
+
 // forward declarations
 class DofConstraints;
 class DofMap;
@@ -56,7 +59,7 @@ class InfFE;
  *
  * \author Benjamin S. Kirk
  * \date 2002-2007
- * \version $Revision: 3391 $
+ * \version $Revision: 3874 $
  */
 
 //-------------------------------------------------------------
@@ -445,7 +448,7 @@ protected:
  *
  * \author Roy Stogner
  * \date 2004
- * \version $Revision: 3391 $
+ * \version $Revision: 3874 $
  */
 
 //-------------------------------------------------------------
@@ -470,7 +473,7 @@ public:
  *
  * \author Roy Stogner
  * \date 2005
- * \version $Revision: 3391 $
+ * \version $Revision: 3874 $
  */
 
 //-------------------------------------------------------------
@@ -505,7 +508,7 @@ public:
  *
  * \author Benjamin S. Kirk
  * \date 2002-2007
- * \version $Revision: 3391 $
+ * \version $Revision: 3874 $
  */
 
 //-------------------------------------------------------------
@@ -530,7 +533,7 @@ public:
  *
  * \author Benjamin S. Kirk
  * \date 2002-2007
- * \version $Revision: 3391 $
+ * \version $Revision: 3874 $
  */
 
 //-------------------------------------------------------------
@@ -555,7 +558,7 @@ public:
  *
  * \author Benjamin S. Kirk
  * \date 2002-2007
- * \version $Revision: 3391 $
+ * \version $Revision: 3874 $
  */
 
 //-------------------------------------------------------------
@@ -597,7 +600,7 @@ public:
  *
  * \author Benjamin S. Kirk
  * \date 2002-2007
- * \version $Revision: 3391 $
+ * \version $Revision: 3874 $
  */
 
 //-------------------------------------------------------------
@@ -829,4 +832,6 @@ FEScalar<Dim>::FEScalar (const FEType& fet) :
 {
 }
 
+} // namespace libMesh
+
 #endif
