@@ -1,4 +1,4 @@
-// $Id: inf_fe.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: inf_fe.h 3874 2010-07-02 21:57:26Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -30,6 +30,9 @@
 
 #include "fe_base.h"
 
+namespace libMesh
+{
+
 
 // forward declarations
 class Elem;
@@ -66,7 +69,7 @@ class FEComputeData;
  *
  * \author Daniel Dreyer
  * \date 2003
- * \version $Revision: 3391 $
+ * \version $Revision: 3874 $
  */
 
 //-------------------------------------------------------------
@@ -89,7 +92,7 @@ protected:
    *
    * \author Daniel Dreyer
    * \date 2003
-   * \version $Revision: 3391 $
+   * \version $Revision: 3874 $
    */
   //-------------------------------------------------------------
   // InfFE::Radial class definition
@@ -181,7 +184,7 @@ protected:
    *
    * \author Daniel Dreyer
    * \date 2003
-   * \version $Revision: 3391 $
+   * \version $Revision: 3874 $
    */
   //-------------------------------------------------------------
   // InfFE::Base class definition
@@ -850,6 +853,8 @@ Real InfFE<Dim,T_radial,T_map>::Radial::decay(const Real v)
 // InfFE::Base class inline members
 
 
+} // namespace libMesh
+
 
 #endif //ifdef LIBMESH_ENABLE_INFINITE_ELEMENTS
 
