@@ -1,4 +1,4 @@
-// $Id: inf_fe_instantiate_1D.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: inf_fe_instantiate_1D.h 3874 2010-07-02 21:57:26Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -36,6 +36,9 @@
 // Local includes
 #include "inf_fe_macro.h"
 
+namespace libMesh
+{
+
 
 /** 
  * Collect all 1D explicit instantiations for class InfFE
@@ -47,5 +50,7 @@ INSTANTIATE_INF_FE(1,CARTESIAN);
 /* INSTANTIATE_INF_FE(1,ELLIPSOIDAL); */
 
 
+} // namespace libMesh
+
 
 #endif
