@@ -1,4 +1,4 @@
-// $Id: elem.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: elem.h 4399 2011-04-22 18:32:39Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -41,10 +41,17 @@
 #include "multi_predicates.h"
 #include "variant_filter_iterator.h"
 
+namespace libMesh
+{
+
 // Forward declarations
 class MeshBase;
 class MeshRefinement;
 class Elem;
+#ifdef LIBMESH_ENABLE_PERIODIC
+class PeriodicBoundaries;
+class PointLocatorBase;
+#endif
 
 /**
  * This is the base class from which all geometric entities
@@ -154,7 +161,7 @@ class Elem : public ReferenceCountedObject<Elem>,
    * nodes, regardless of how those nodes might be numbered local
    * to the elements.
    */
-  virtual bool operator == (const DofObject& rhs) const;
+  bool operator == (const Elem& rhs) const;
 
   /**
    * @returns a pointer to the \f$ i^{th} \f$ neighbor of this element.
@@ -165,6 +172,28 @@ class Elem : public ReferenceCountedObject<Elem>,
    */
   Elem* neighbor (const unsigned int i) const;
 
+#ifdef LIBMESH_ENABLE_PERIODIC  
+  /**
+   * @returns a pointer to the \f$ i^{th} \f$ neighbor of this element
+   * for interior elements.  If an element is on a periodic
+   * boundary, it will return a corresponding element on the opposite
+   * side. 
+   */
+  Elem* topological_neighbor (const unsigned int i,
+                              const MeshBase& mesh,
+                              const PointLocatorBase& point_locator,
+                              PeriodicBoundaries* pb) const;
+
+  /**
+   * @return \p true if the element \p elem in question is a neighbor or
+   * topological neighbor of this element, \p false otherwise.
+   */
+  bool has_topological_neighbor (const Elem* elem,
+                                 const MeshBase& mesh,
+                                 const PointLocatorBase& point_locator,
+                                 PeriodicBoundaries* pb) const;
+#endif   
+
   /**
    * Assigns \p n as the \f$ i^{th} \f$ neighbor.
    */
@@ -210,12 +239,25 @@ class Elem : public ReferenceCountedObject<Elem>,
    */
   bool contains_vertex_of(const Elem *e) const;
 
+  /**
+   * This function returns true iff an edge of \p e is contained in
+   * this element.  (Internally, this is done by checking whether at
+   * least two vertices of \p e are contained in this element).
+   */
+  bool contains_edge_of(const Elem *e) const;
+
   /**
    * This function finds all elements which
    * touch the current element at any point
    */
   void find_point_neighbors(std::set<const Elem *> &neighbor_set) const;
 
+  /**
+   * This function finds all elements which touch the current element
+   * at any edge (more precisely, at at least two points).
+   */
+  void find_edge_neighbors(std::set<const Elem *> &neighbor_set) const;
+
   /**
    * Resets this element's neighbors' appropriate neighbor pointers
    * and its parent's and children's appropriate pointers
@@ -375,7 +417,7 @@ class Elem : public ReferenceCountedObject<Elem>,
    * you want the full-ordered face (i.e. a 9-noded quad face for a 27-noded
    * hexahedral) use the build_side method.
    */
-  virtual AutoPtr<DofObject> side (const unsigned int i) const = 0;
+  virtual AutoPtr<Elem> side (const unsigned int i) const = 0;
   
   /**
    * Creates an element coincident with side \p i. The element returned is
@@ -455,8 +497,12 @@ class Elem : public ReferenceCountedObject<Elem>,
   /**
    * @returns true if the point p is contained in this element, 
    * false otherwise.
+   *
+   * Since we are doing floating point comparisons here the parameter
+   * \p tol can be specified to indicate a tolerance.  For example,
+   * \f$ \xi \le 1 \f$  becomes \f$ \xi \le 1 + \epsilon \f$. 
    */
-  virtual bool contains_point (const Point& p) const;
+  virtual bool contains_point (const Point& p, Real tol=TOLERANCE) const;
 
   /**
    * @returns true iff the element map is definitely affine (i.e. the same at
@@ -518,7 +564,7 @@ class Elem : public ReferenceCountedObject<Elem>,
   Elem* parent ();
 
   /**
-   * @Sets the pointer to the element's parent.
+   * Sets the pointer to the element's parent.
    * Dangerous to use in high-level code.
    */
   void set_parent (Elem *p);
@@ -675,6 +721,12 @@ class Elem : public ReferenceCountedObject<Elem>,
    */
   void add_child (Elem* elem, unsigned int c);
 
+  /**
+   * Replaces the child pointer at the specified index in the array of
+   * children of this element.
+   */
+  void replace_child (Elem* elem, unsigned int c);
+
   /**
    * Fills the vector \p family with the children of this element,
    * recursively.  So, calling this method on a twice-refined element
@@ -1058,17 +1110,20 @@ inline
 Elem::Elem(const unsigned int nn,
 	   const unsigned int ns,
 	   Elem* p) :
-  _parent(p)
+  _nodes(NULL),
+  _neighbors(NULL),
+  _parent(p),
 #ifdef LIBMESH_ENABLE_AMR
-  , _p_level(0)
+  _children(NULL),
+  _rflag(Elem::DO_NOTHING),
+  _pflag(Elem::DO_NOTHING),
+  _p_level(0),
 #endif
+  _sbd_id(0)
 {
-  this->subdomain_id() = 0;
   this->processor_id() = DofObject::invalid_processor_id;
   
   // Initialize the nodes data structure
-  _nodes = NULL;
-  
   if (nn != 0)
     {
       _nodes = new Node*[nn]; 
@@ -1078,8 +1133,6 @@ Elem::Elem(const unsigned int nn,
     }
   
   // Initialize the neighbors data structure
-  _neighbors = NULL;
-  
   if (ns != 0)
     {
       _neighbors = new Elem*[ns]; 
@@ -1096,18 +1149,8 @@ Elem::Elem(const unsigned int nn,
     }  
 
 #ifdef LIBMESH_ENABLE_AMR
-  
-  _children = NULL;
-
-  this->set_refinement_flag(Elem::DO_NOTHING);
-
-  this->set_p_refinement_flag(Elem::DO_NOTHING);
-
   if (this->parent())
     this->set_p_level(parent()->p_level());
-  else
-    this->set_p_level(0);
-
 #endif
 }
 
@@ -1363,10 +1406,14 @@ bool Elem::has_ancestor_children() const
 
 
 
-#ifdef LIBMESH_ENABLE_AMR
 inline
-bool Elem::is_ancestor_of(const Elem *descendant) const
+bool Elem::is_ancestor_of(const Elem *
+#ifdef LIBMESH_ENABLE_AMR
+			  descendant
+#endif
+			  ) const
 {
+#ifdef LIBMESH_ENABLE_AMR
   const Elem *e = descendant;
   while (e)
     {
@@ -1374,10 +1421,6 @@ bool Elem::is_ancestor_of(const Elem *descendant) const
         return true;
       e = e->parent();
     }
-#else
-inline
-bool Elem::is_ancestor_of(const Elem *) const
-{
 #endif
   return false;
 }
@@ -1516,8 +1559,8 @@ unsigned int Elem::which_child_am_i (const Elem* e) const
     if (this->child(c) == e)
       return c;
 
-  std::cerr << "ERROR:  which_child_am_i() was called with a non-child!" 
-	    << std::endl;
+  libMesh::err << "ERROR:  which_child_am_i() was called with a non-child!" 
+	        << std::endl;
 
   libmesh_error();
 
@@ -1540,10 +1583,10 @@ void Elem::set_refinement_flag(RefinementState rflag)
 #ifdef DEBUG
   if (rflag != static_cast<RefinementState>(static_cast<unsigned char>(rflag)))
     {
-      std::cerr << "ERROR: unsigned char too small to hold Elem::_rflag!"
-		<< std::endl
-		<< "Recompile with Elem:_*flag set to something bigger!"
-		<< std::endl;
+      libMesh::err << "ERROR: unsigned char too small to hold Elem::_rflag!"
+		    << std::endl
+		    << "Recompile with Elem:_*flag set to something bigger!"
+		    << std::endl;
       libmesh_error();
     }
 #endif
@@ -1567,10 +1610,10 @@ void Elem::set_p_refinement_flag(RefinementState pflag)
 #ifdef DEBUG
   if (pflag != static_cast<RefinementState>(static_cast<unsigned char>(pflag)))
     {
-      std::cerr << "ERROR: unsigned char too small to hold Elem::_pflag!"
-		<< std::endl
-		<< "Recompile with Elem:_*flag set to something bigger!"
-		<< std::endl;
+      libMesh::err << "ERROR: unsigned char too small to hold Elem::_pflag!"
+		    << std::endl
+		    << "Recompile with Elem:_*flag set to something bigger!"
+		    << std::endl;
       libmesh_error();
     }
 #endif
@@ -1604,10 +1647,10 @@ void Elem::set_p_level(unsigned int p)
 #ifdef DEBUG
   if (p != static_cast<unsigned int>(static_cast<unsigned char>(p)))
     {
-      std::cerr << "ERROR: unsigned char too small to hold Elem::_p_level!"
-		<< std::endl
-		<< "Recompile with Elem:_p_level set to something bigger!"
-		<< std::endl;
+      libMesh::err << "ERROR: unsigned char too small to hold Elem::_p_level!"
+		    << std::endl
+		    << "Recompile with Elem:_p_level set to something bigger!"
+		    << std::endl;
       libmesh_error();
     }
 #endif
@@ -1907,6 +1950,7 @@ public:
   SideIter(const unsigned int side_number,
 	   Elem* parent)
     : _side_number(side_number),
+      _side(),
       _side_ptr(NULL),
       _parent(parent)
   {}
@@ -1915,6 +1959,7 @@ public:
   // Empty constructor.
   SideIter()
     : _side_number(libMesh::invalid_uint),
+      _side(),
       _side_ptr(NULL),
       _parent(NULL)
   {}
@@ -1923,6 +1968,8 @@ public:
   // Copy constructor
   SideIter(const SideIter& other)
     : _side_number(other._side_number),
+      _side(),
+      _side_ptr(NULL),
       _parent(other._parent)
   {}
 
@@ -2047,5 +2094,7 @@ variant_filter_iterator<Elem::Predicate,
 
 
 
+} // namespace libMesh
+
 
 #endif // end #ifndef __elem_h__
