@@ -1,4 +1,4 @@
-// $Id: elem_range.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: elem_range.h 3874 2010-07-02 21:57:26Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -29,7 +29,12 @@
 #include "elem.h"
 #include "stored_range.h"
 
+namespace libMesh
+{
+
 typedef StoredRange<MeshBase::element_iterator,             Elem*>      ElemRange;
 typedef StoredRange<MeshBase::const_element_iterator, const Elem*> ConstElemRange;
 
+} // namespace libMesh
+
 #endif // end #ifndef __elem_range_h__
