@@ -1,4 +1,4 @@
-// $Id: node_range.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: node_range.h 3874 2010-07-02 21:57:26Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -29,7 +29,12 @@
 #include "node.h"
 #include "stored_range.h"
 
+namespace libMesh
+{
+
 typedef StoredRange<MeshBase::node_iterator,             Node*>      NodeRange;
 typedef StoredRange<MeshBase::const_node_iterator, const Node*> ConstNodeRange;
 
+} // namespace libMesh
+
 #endif // end #ifndef __node_range_h__
