@@ -1,18 +1,18 @@
-// $Id: exodusII_io_helper.h 3462 2009-09-15 17:00:11Z friedmud $
+// $Id: exodusII_io_helper.h 4172 2011-01-17 21:56:25Z jasondhales $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
-  
+
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-  
+
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-  
+
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -24,11 +24,15 @@
 
 #ifdef LIBMESH_HAVE_EXODUS_API
 
-#include <vector>
+#include <iostream>
 #include <string>
+#include <vector>
 
 #include "mesh_base.h"
 
+namespace libMesh
+{
+
 
 namespace exII {
   extern "C" {
@@ -52,12 +56,13 @@ public:
    * the class.  Also allows you to set the verbosity level to v=1
    * (on) or v=0 (off).
    */
-  ExodusII_IO_Helper(bool v=false) : 
-    comp_ws(sizeof(double)),
+  ExodusII_IO_Helper(bool v=false) :
+    comp_ws(sizeof(Real)),
     io_ws(0),
     ex_id(0),
     ex_err(0),
     num_dim(0),
+    num_globals(0),
     num_nodes(0),
     num_elem(0),
     num_elem_blk(0),
@@ -74,7 +79,9 @@ public:
     ret_char(0),
     num_time_steps(0),
     _created(false),
-    _verbose(v)
+    _verbose(v),
+    _elem_vars_initialized(false),
+    _global_vars_initialized(false)
 
   {
     title.resize(MAX_LINE_LENGTH+1);
@@ -94,25 +101,31 @@ public:
    * false otherwise.
    */
   bool created();
-  
+
   /**
    * Get/set flag telling whether message printing is on or off.
    */
   void verbose (bool set_verbosity);
-  
+
   /**
    * @returns the \p ExodusII
    * mesh dimension.
    */
   int get_num_dim()                const { return num_dim; }
 
+  /**
+   * @returns the total number of
+   * global variables.
+   */
+  int get_num_globals()            const { return num_globals; }
+
   /**
    * @returns the total number of
    * nodes in the \p ExodusII mesh.
    */
   int get_num_nodes()              const { return num_nodes; }
 
-  
+
   /**
    * @returns the total number of
    * elements in the \p ExodusII mesh.
@@ -179,7 +192,7 @@ public:
    * elements in the sideset.
    */
   int get_num_sides_per_set(int i) const { return num_sides_per_set[i]; }
-  
+
   /**
    * For a single nodeset,
    * returns the total number of
@@ -197,10 +210,10 @@ public:
   //     int get_elem_list(int i)         const { return elem_list[i]; }
 
   /**
-   * @return a constant reference to the \p elem_list.   
+   * @return a constant reference to the \p elem_list.
    */
   const std::vector<int>& get_elem_list() const { return elem_list; }
-  
+
   //     /**
   //      * @returns the \f$ i^{th} \f$ entry in
   //      * the side list.  This is
@@ -212,12 +225,12 @@ public:
   //     int get_side_list(int i)         const { return side_list[i]; }
 
   /**
-   * @return a constant reference to the \p side_list.   
+   * @return a constant reference to the \p side_list.
    */
   const std::vector<int>& get_side_list() const { return side_list; }
 
   /**
-   * @return a constant reference to the \p node_list.   
+   * @return a constant reference to the \p node_list.
    */
   const std::vector<int>& get_node_list() const { return node_list; }
 
@@ -234,10 +247,10 @@ public:
   //     int get_id_list(int i)         const { return id_list[i]; }
 
   /**
-   * @return a constant reference to the \p id_list.   
+   * @return a constant reference to the \p id_list.
    */
   const std::vector<int>& get_id_list() const { return id_list; }
-  
+
   /**
    * @returns the current
    * element type.  Note:
@@ -252,19 +265,19 @@ public:
    * @returns the \f$ i^{th} \f$
    * node's x-coordinate.
    */
-  double get_x(int i) const { return x[i]; }
+  Real get_x(int i) const { return x[i]; }
 
   /**
    * @returns the \f$ i^{th} \f$
    * node's y-coordinate.
    */
-  double get_y(int i) const { return y[i]; }
+  Real get_y(int i) const { return y[i]; }
 
   /**
    * @returns the \f$ i^{th} \f$
    * node's z-coordinate.
    */
-  double get_z(int i) const { return z[i]; }
+  Real get_z(int i) const { return z[i]; }
 
   /**
    * Opens an \p ExodusII mesh
@@ -304,12 +317,12 @@ public:
    * from the \p ExodusII mesh file.
    */
   void read_node_num_map();
-  
+
   /**
-   * Prints the nodal information
-   * to \p std::cout.
+   * Prints the nodal information,
+   * by default to \p libMesh::out.
    */
-  void print_nodes();
+  void print_nodes(std::ostream &out = libMesh::out);
 
   /**
    * Reads information for
@@ -323,7 +336,7 @@ public:
    * for the given block.
    */
   int get_block_id(int block);
-  
+
   /**
    * Reads all of the element
    * connectivity for
@@ -331,7 +344,7 @@ public:
    * \p ExodusII mesh file.
    */
   void read_elem_in_block(int block);
- 
+
   /**
    * Reads the optional \p node_num_map
    * from the \p ExodusII mesh file.
@@ -393,19 +406,19 @@ public:
    */
   int inquire(int req_info, std::string error_msg="");
 
-    
+
   // For reading solutions:
   /*
    * Returns an array containing the timesteps in the file
    */
-  const std::vector<double>& get_time_steps();
+  const std::vector<Real>& get_time_steps();
 
 
   /*
    * Number of Nodal variables defined.
    */
   int get_num_nodal_vars(){ return num_nodal_vars; }
-    
+
 
   /*
    * Returns an array containing the nodal var names in the file
@@ -416,7 +429,7 @@ public:
    * Returns an array containing the nodal variable values
    * at the specified time
    */
-  const std::vector<double>& get_nodal_var_values(std::string nodal_var_name, int time_step);
+  const std::vector<Real>& get_nodal_var_values(std::string nodal_var_name, int time_step);
 
   // For Writing Solutions
   /**
@@ -431,17 +444,33 @@ public:
    */
   void initialize(std::string title, const MeshBase & mesh);
 
+  /**
+   * Initializes the Exodus file
+   */
+  void initialize_discontinuous(std::string title, const MeshBase & mesh);
+
   /**
    * Writes the nodal coordinates contained in "mesh"
    */
   void write_nodal_coordinates(const MeshBase & mesh);
 
+  /**
+   * Writes the nodal coordinates contained in "mesh"
+   */
+  void write_nodal_coordinates_discontinuous(const MeshBase & mesh);
+
   /**
    * Writes the elements contained in "mesh"
    * FIXME: This only works for Mesh's having a single type of element!
    */
   void write_elements(const MeshBase & mesh);
 
+  /**
+   * Writes the elements contained in "mesh"
+   * FIXME: This only works for Mesh's having a single type of element!
+   */
+  void write_elements_discontinuous(const MeshBase & mesh);
+
   /**
    * Writes the sidesets contained in "mesh"
    */
@@ -452,21 +481,46 @@ public:
    */
   void write_nodesets(const MeshBase & mesh);
 
+  /**
+   * Sets up the nodal variables
+   */
+  void initialize_element_variables(std::vector<std::string> names);
+
   /**
    * Sets up the nodal variables
    */
   void initialize_nodal_variables(std::vector<std::string> names);
 
+  /**
+   * Sets up the global variables
+   */
+  void initialize_global_variables(const std::vector<std::string> & names);
+
   /**
    * Writes the time for the timestep
    */
-  void write_timestep(int timestep, double time);
+  void write_timestep(int timestep, Real time);
+
+  /**
+   * Writes the vector of values to the element variables.
+   */
+  void write_element_values(const MeshBase & mesh, const std::vector<Number> & values, int timestep);
 
   /**
    * Writes the vector of values to a nodal variable.
    */
   void write_nodal_values(int var_id, const std::vector<Number> & values, int timestep);
 
+  /**
+   * Writes the vector of information records.
+   */
+  void write_information_records(const std::vector<std::string> & records);
+
+  /**
+   * Writes the vector of global variables.
+   */
+  void write_global_values(const std::vector<Number> & values, int timestep);
+
 
 
   /**
@@ -475,7 +529,7 @@ public:
    * maps and name conversions.  It's defined below.
    */
   class Conversion;
-  
+
   /**
    * This is the \p ExodusII_IO_Helper ElementMap class.
    * It contains constant maps between the \p ExodusII naming/numbering
@@ -484,10 +538,10 @@ public:
    */
   class ElementMaps;
 
-  
+
   //private:
 
-  
+
   /**
    * All of the \p ExodusII
    * API functions return
@@ -502,16 +556,14 @@ public:
 
   /**
    * Prints the message defined
-   * in \p msg to \p std::cout.
-   * Can be turned off if
+   * in \p msg. Can be turned off if
    * verbosity is set to 0.
    */
   void message(const std::string msg);
 
   /**
    * Prints the message defined
-   * in \p msg to \p std::cout
-   * and appends the number
+   * in \p msg, and appends the number
    * \p i to the end of the
    * message.  Useful for
    * printing messages in loops.
@@ -525,6 +577,7 @@ public:
   int   ex_id;                         // File identification flag
   int   ex_err;                        // General error flag
   int   num_dim;                       // Number of dimensions in the mesh
+  int   num_globals;                   // Number of global variables
   int   num_nodes;                     // Total number of nodes in the mesh
   int   num_elem;                      // Total number of elements in the mesh
   int   num_elem_blk;                  // Total number of element blocks
@@ -545,16 +598,16 @@ public:
   std::vector<int> num_df_per_set;     // Number of distribution factors per set
   std::vector<int> num_node_df_per_set;// Number of distribution factors per set
   std::vector<int> elem_list;          // List of element numbers in all sidesets
-  std::vector<int> side_list;          // Side (face/edge) number actually on the boundary 
-  std::vector<int> node_list;          // Node number actually on the boundary 
+  std::vector<int> side_list;          // Side (face/edge) number actually on the boundary
+  std::vector<int> node_list;          // Node number actually on the boundary
   std::vector<int> id_list;            // Side (face/edge) id number
   std::vector<int> node_num_map;       // Optional mapping from internal [0,num_nodes) to arbitrary indices
   std::vector<int> elem_num_map;       // Optional mapping from internal [0,num_elem) to arbitrary indices
   float ex_version;                    // Version of Exodus you are using
   float ret_float;                     // Generic float returned by ex_inquire
-  std::vector<double> x;               // x locations of node points
-  std::vector<double> y;               // y locations of node points
-  std::vector<double> z;               // z locations of node points
+  std::vector<Real> x;                 // x locations of node points
+  std::vector<Real> y;                 // y locations of node points
+  std::vector<Real> z;                 // z locations of node points
   char    ret_char;                    // Generic char returned by ex_inquire
   // Use vectors of char to emulate char*'s
   std::vector<char> title;             //  Problem title
@@ -563,13 +616,15 @@ public:
   // Maps libMesh element numbers to Exodus element numbers
   // gets filled in when write_elements gets called
   std::map<int, int> libmesh_elem_num_to_exodus;
-    
+
   //Solution Data
   int num_time_steps;
-  std::vector<double> time_steps;
+  std::vector<Real> time_steps;
   int num_nodal_vars;
   std::vector<std::string> nodal_var_names;
-  std::vector<double> nodal_var_values;
+  std::vector<Real> nodal_var_values;
+
+  int num_elem_vars;
 
   // A pair of containers used to emulate a char** data
   // structure without having to worry about dynamic memory
@@ -580,6 +635,8 @@ public:
  protected:
   bool _created; // This flag gets set after the the create() function has been successfully called.
   bool _verbose; // On/Off message flag
+  bool _elem_vars_initialized; // True once the elem vars are initialized
+  bool _global_vars_initialized; // True once the global vars are initialized
 };
 
 
@@ -597,7 +654,7 @@ public:
    * Constructor.  Initializes the const private member
    * variables.
    */
-  Conversion(const int* nm, const int* sm, const int* ism, const ElemType ct, std::string ex_type) 
+  Conversion(const int* nm, const int* sm, const int* ism, const ElemType ct, std::string ex_type)
     : node_map(nm),       // Node map for this element
       side_map(sm),
       inverse_side_map(ism),
@@ -638,13 +695,13 @@ public:
    */
   std::string exodus_elem_type() const { return exodus_type; };
 
-    
+
 private:
   /**
    * Pointer to the node map for this element.
    */
   const int* node_map;
-      
+
   /**
    * Pointer to the side map for this element.
    */
@@ -692,35 +749,35 @@ public:
    * Use this map for bi-linear
    * quadrilateral elements in 2D.
    */
-  static const int quad4_node_map[4]; 
+  static const int quad4_node_map[4];
 
   /**
    * The Quad8 node map.
    * Use this map for serendipity
    * quadrilateral elements in 2D.
    */
-  static const int quad8_node_map[8]; 
+  static const int quad8_node_map[8];
 
   /**
    * The Quad9 node map.
    * Use this map for bi-quadratic
    * quadrilateral elements in 2D.
    */
-  static const int quad9_node_map[9]; 
+  static const int quad9_node_map[9];
 
   /**
    * The Tri3 node map.
    * Use this map for linear
    * triangles in 2D.
    */
-  static const int tri3_node_map[3];     
+  static const int tri3_node_map[3];
 
   /**
    * The Tri6 node map.
    * Use this map for quadratic
    * triangular elements in 2D.
    */
-  static const int tri6_node_map[6];     
+  static const int tri6_node_map[6];
 
   /**
    * 2D edge maps
@@ -729,27 +786,27 @@ public:
   /**
    * Maps the Exodus edge numbering for triangles.
    * Useful for reading sideset information.
-   */ 
-  static const int tri_edge_map[3]; 
+   */
+  static const int tri_edge_map[3];
 
   /**
    * Maps the Exodus edge numbering for quadrilaterals.
    * Useful for reading sideset information.
    */
-  static const int quad_edge_map[4]; 
+  static const int quad_edge_map[4];
 
   /**
    * Maps the Exodus edge numbering for triangles.
    * Useful for writing sideset information.
-   */ 
-  static const int tri_inverse_edge_map[3]; 
+   */
+  static const int tri_inverse_edge_map[3];
 
   /**
    * Maps the Exodus edge numbering for quadrilaterals.
    * Useful for writing sideset information.
    */
   static const int quad_inverse_edge_map[4];
-      
+
   /**
    * 3D maps.  These define
    * mappings from ExodusII-formatted
@@ -761,88 +818,88 @@ public:
    * Use this map for bi-linear
    * hexahedral elements in 3D.
    */
-  static const int hex8_node_map[8]; 
+  static const int hex8_node_map[8];
 
   /**
    * The Hex20 node map.
    * Use this map for serendipity
    * hexahedral elements in 3D.
    */
-  static const int hex20_node_map[20]; 
+  static const int hex20_node_map[20];
 
   /**
    * The Hex27 node map.
    * Use this map for bi-quadratic
    * hexahedral elements in 3D.
    */
-  static const int hex27_node_map[27]; 
+  static const int hex27_node_map[27];
 
   /**
    * The Tet4 node map.
    * Use this map for linear
    * tetrahedral elements in 3D.
    */
-  static const int tet4_node_map[4]; 
+  static const int tet4_node_map[4];
 
   /**
    * The Tet10 node map.
    * Use this map for quadratic
    * tetrahedral elements in 3D.
    */
-  static const int tet10_node_map[10]; 
+  static const int tet10_node_map[10];
 
   /**
    * The Prism6 node map.
    */
-  static const int prism6_node_map[6]; 
+  static const int prism6_node_map[6];
 
   /**
    * The Prism15 node map.
    * Use this map for "serendipity" prisms in 3D.
    */
-  static const int prism15_node_map[15]; 
+  static const int prism15_node_map[15];
 
   /**
    * The Prism18 node map.
    */
-  static const int prism18_node_map[18]; 
+  static const int prism18_node_map[18];
 
   /**
    * The Pyramid5 node map.
    * Use this map for linear
    * pyramid elements in 3D.
    */
-  static const int pyramid5_node_map[5]; 
+  static const int pyramid5_node_map[5];
 
 
   /**
    * 3D face maps.  Are these ever used for anything?
    */
-      
+
   /**
    * Maps the Exodus face numbering for general hexahedrals.
    * Useful for reading sideset information.
    */
   static const int hex_face_map[6];
-      
+
   /**
    * Maps the Exodus face numbering for 27-noded hexahedrals.
    * Useful for reading sideset information.
    */
   static const int hex27_face_map[6];
-      
+
   /**
    * Maps the Exodus face numbering for general tetrahedrals.
    * Useful for reading sideset information.
    */
   static const int tet_face_map[4];
-      
+
   /**
    * Maps the Exodus face numbering for general prisms.
    * Useful for reading sideset information.
    */
   static const int prism_face_map[5];
-      
+
   /**
    * Maps the Exodus face numbering for general pyramids.
    * Useful for reading sideset information.
@@ -854,25 +911,25 @@ public:
    * Useful for writing sideset information.
    */
   static const int hex_inverse_face_map[6];
-      
+
   /**
    * Maps the Exodus face numbering for 27-noded hexahedrals.
    * Useful for writing sideset information.
    */
   static const int hex27_inverse_face_map[6];
-      
+
   /**
    * Maps the Exodus face numbering for general tetrahedrals.
    * Useful for writing sideset information.
    */
   static const int tet_inverse_face_map[4];
-      
+
   /**
    * Maps the Exodus face numbering for general prisms.
    * Useful for writing sideset information.
    */
   static const int prism_inverse_face_map[5];
-      
+
   /**
    * Maps the Exodus face numbering for general pyramids.
    * Useful for writing sideset information.
@@ -884,7 +941,7 @@ public:
    * @returns a conversion object given an element type name.
    */
   ExodusII_IO_Helper::Conversion assign_conversion(const std::string type_str);
-      
+
   /**
    * @returns a conversion object given an element type.
    */
@@ -892,6 +949,8 @@ public:
 };
 
 
+} // namespace libMesh
+
 #endif // LIBMESH_HAVE_EXODUS_API
 
 #endif // #ifndef __exodusII_io_helper_h__
