@@ -1,4 +1,4 @@
-// $Id: mesh_base.h 3454 2009-09-03 16:59:40Z roystgnr $
+// $Id: mesh_base.h 4402 2011-04-22 23:10:19Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -27,23 +27,25 @@
 // C++ Includes   -----------------------------------
 #include <string>
 
-// forward declarations
-class Elem;
-class Node;
-class Point;
-class BoundaryInfo;
-class PointLocatorBase;
-class MeshData;
-
 // Local Includes -----------------------------------
 #include "auto_ptr.h"
 #include "dof_object.h" // for invalid_processor_id
 #include "enum_elem_type.h"
 #include "libmesh_common.h"
 #include "multi_predicates.h"
-#include "partitioner.h"
+#include "partitioner.h" // AutoPtr needs a real declaration
+#include "point_locator_base.h"
 #include "variant_filter_iterator.h"
 
+namespace libMesh
+{
+
+// forward declarations
+class Elem;
+class Node;
+class Point;
+class BoundaryInfo;
+class MeshData;
 
 
 
@@ -59,8 +61,8 @@ class MeshData;
  * mesh to disk in various formats.
  *
  * \author  Benjamin S. Kirk
- * \date    $Date: 2009-09-03 11:59:40 -0500 (Thu, 03 Sep 2009) $
- * \version $Revision: 3454 $
+ * \date    $Date: 2011-04-22 18:10:19 -0500 (Fri, 22 Apr 2011) $
+ * \version $Revision: 4402 $
  */
 
 
@@ -71,9 +73,11 @@ class MeshBase
 public:
 
   /**
-   * Constructor.  Requires \p d, the dimension of the mesh.
+   * Constructor.  Takes \p dim, the dimension of the mesh.
+   * The mesh dimension can be changed (and may automatically be
+   * changed by mesh generation/loading) later.
    */
-  MeshBase (unsigned int d);
+  MeshBase (unsigned int dim=1);
   
   /**
    * Copy-constructor.
@@ -122,30 +126,32 @@ public:
   { return true; }
   
   /**
-   * @gathers all elements and nodes of the mesh onto
+   * Gathers all elements and nodes of the mesh onto
    * every processor
    */
   virtual void allgather () {}
   
   /**
-   * @when supported, deletes all nonlocal elements of the mesh
+   * When supported, deletes all nonlocal elements of the mesh
    * except for "ghosts" which touch a local element, and deletes
    * all nodes which are not part of a local or ghost element
    */
   virtual void delete_remote_elements () {}
   
   /**
-   * Returns the logical dimension of the mesh.
+   * @returns the logical dimension of the mesh; i.e. the manifold
+   * dimension of the elements in the mesh.  If we ever support
+   * multi-dimensional meshes (e.g. hexes and quads in the same mesh)
+   * then this will return the largest such dimension.
    */
   unsigned int mesh_dimension () const
   { return static_cast<unsigned int>(_dim); }
   
   /**
    * Resets the logical dimension of the mesh.
-   * Should only be called on an empty mesh.
    */
   void set_mesh_dimension (unsigned int d)
-  { libmesh_assert(!this->n_elem()); _dim = d; }
+  { _dim = d; }
   
   /**
    * Returns the spatial dimension of the mesh.  Note that this is
@@ -382,6 +388,15 @@ public:
    */
   virtual void renumber_nodes_and_elements () = 0;    
 
+    /**
+     * There is no reason for a user to ever call this function.
+     *
+     * This function restores a previously broken element/node numbering such that
+     * \p mesh.node(n)->id() == n. 
+     */
+  virtual void fix_broken_node_and_element_numbering () = 0;
+  
+
 #ifdef LIBMESH_ENABLE_AMR
   /**
    * Delete subactive (i.e. children of coarsened) elements.
@@ -402,13 +417,30 @@ public:
    * is called from Mesh::read after reading an xda file.  It prevents
    * the renumbering of nodes and elements.  In general, leave this at
    * the default value of false.
+   *
+   * skip_renumber is currently set to TRUE to work around an I/O bug
    */
-  void prepare_for_use (const bool skip_renumber_nodes_and_elements=false);
+  void prepare_for_use (const bool skip_renumber_nodes_and_elements=true);
   
   /**
    * Call the default partitioner (currently \p metis_partition()).
    */
-  void partition (const unsigned int n_parts=libMesh::n_processors());
+  virtual void partition (const unsigned int n_parts=libMesh::n_processors());
+
+  /**
+   * If true is passed in then this mesh will no longer be (re)partitioned.
+   * It would probably be a bad idea to call this on a Serial Mesh _before_
+   * the first partitioning has happened... because no elements would get assigned
+   * to your processor pool.
+   *
+   * Note that turning on skip_partitioning() can have adverse effects on your
+   * performance when using AMR... ie you could get large load imbalances.
+   *
+   * However you might still want to use this if the communication and computation
+   * of the rebalance and repartition is too high for your application.
+   */
+  void skip_partitioning(bool skip) { _skip_partitioning = skip; }
+  bool skip_partitioning() { return _skip_partitioning; }
   
   /**
    * Returns the number of subdomains in the global mesh. Subdomains correspond
@@ -417,8 +449,7 @@ public:
    * physical processes are important.  The subdomain mapping is independent
    * from the parallel decomposition.
    */
-  unsigned int n_subdomains () const
-  { return _n_sbd; }
+  unsigned int n_subdomains () const;
 
   /**
    * Returns the number of partitions which have been defined via
@@ -451,12 +482,12 @@ public:
   /**
    * Prints relevant information about the mesh.
    */
-  void print_info (std::ostream& os=std::cout) const;
+  void print_info (std::ostream& os=libMesh::out) const;
 
   /**
    * Equivalent to calling print_info() above, but now you can write:
    * Mesh mesh;
-   * std::cout << mesh << std::endl;
+   * libMesh::out << mesh << std::endl;
    */
   friend std::ostream& operator << (std::ostream& os, const MeshBase& m);
 
@@ -525,17 +556,35 @@ public:
   unsigned int recalculate_n_partitions();
 
   /**
-   * \p returns a reference to a \p PointLocatorBase object for this mesh.
-   * 
+   * \p returns a pointer to a \p PointLocatorBase object for this
+   * mesh, constructing a master PointLocator first if necessary.
+   * This should never be used in threaded or non-parallel_only code,
+   * and so is deprecated.
    */
-  const PointLocatorBase & point_locator () const;
+  const PointLocatorBase& point_locator () const;
+
+  /**
+   * \p returns a pointer to a subordinate \p PointLocatorBase object
+   * for this mesh, constructing a master PointLocator first if
+   * necessary.  This should not be used in threaded or
+   * non-parallel_only code unless the master has already been
+   * constructed.
+   */
+  AutoPtr<PointLocatorBase> sub_point_locator () const;
 
   /**
    * Releases the current \p PointLocator object.
    */
   void clear_point_locator ();
 
-
+  /**
+   * Verify id and processor_id consistency of our elements and
+   * nodes containers.
+   * Calls libmesh_assert() on each possible failure.
+   * Currently only implemented on ParallelMesh; a serial data
+   * structure is much harder to get out of sync.
+   */
+  virtual void libmesh_assert_valid_parallel_ids() const {}
 
 public:
 
@@ -665,13 +714,6 @@ public:
 
 
   
-  /**
-   * Returns a writeable reference to the number of subdomains.
-   */
-  unsigned int& set_n_subdomains ()
-  { return _n_sbd; }
-  
-  
 protected:
 
 
@@ -685,16 +727,6 @@ protected:
   unsigned int& set_n_partitions ()
   { return _n_parts; }
   
-  /**
-   * The number of subdomains the mesh has.
-   * **NOTE** Not to be confused with the number of paritions!
-   * The definition of subdomain can be anything the user wants,
-   * e.g. a solid region bounded by a liquid region could be
-   * referred to as subdomains 1 and 2, but those subdomains
-   * could be partitioned over many processors.
-   */
-  unsigned int _n_sbd;
-
   /**
    * The number of partitions the mesh has.  This is set by
    * the partitioners, and may not be changed directly by
@@ -732,6 +764,11 @@ protected:
    * can be replaced by the user through the partitioner() accessor.
    */
   AutoPtr<Partitioner> _partitioner;
+
+  /**
+   * If this is true then no partitioning should be done.
+   */
+  bool _skip_partitioning;
   
   /**
    * The partitioner class is a friend so that it can set
@@ -807,7 +844,7 @@ variant_filter_iterator<MeshBase::Predicate,
 			    Elem* const&,
 			    Elem* const*>(rhs)
   {
-    // std::cout << "Called element_iterator conversion-to-const ctor." << std::endl;
+    // libMesh::out << "Called element_iterator conversion-to-const ctor." << std::endl;
   }
 };
 
@@ -867,9 +904,11 @@ variant_filter_iterator<MeshBase::Predicate,
 			    Node* const &,
 			    Node* const *>(rhs)
   {
-    // std::cout << "Called node_iterator conversion-to-const ctor." << std::endl;
+    // libMesh::out << "Called node_iterator conversion-to-const ctor." << std::endl;
   }
 };
 
 
+} // namespace libMesh
+
 #endif
