@@ -1,4 +1,4 @@
-// $Id: mesh_data.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: mesh_data.h 3874 2010-07-02 21:57:26Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -30,6 +30,9 @@
 #include "libmesh.h"
 #include "enum_xdr_mode.h"
 
+namespace libMesh
+{
+
 
 // Forward Declarations
 class Node;
@@ -177,7 +180,7 @@ public:
   /**
    * Prints relevant information about the mesh.
    */
-  void print_info (std::ostream& os=std::cout) const;
+  void print_info (std::ostream& os=libMesh::out) const;
 
   /**
    * Same as above, but allows you to use the stream syntax.
@@ -210,7 +213,7 @@ public:
   const std::vector<Number>& get_data (const Node* node) const;
 
   /**
-   * @Sets all the data values associated with 
+   * Sets all the data values associated with 
    * the node \p node, overwriting any existing vector
    */
   void set_data (const Node* node, const std::vector<Number>& val);
@@ -288,7 +291,7 @@ public:
   const std::vector<Number>& get_data (const Elem* elem) const;
 
   /**
-   * @Sets all the data values associated with 
+   * Sets all the data values associated with 
    * the element \p elem, overwriting any existing vector
    */
   void set_data (const Elem* elem, const std::vector<Number> &val);
@@ -961,7 +964,7 @@ const std::vector<Number>& MeshData::get_data (const Elem* elem) const
 #ifdef DEBUG
   if (pos == _elem_data.end())
     {
-      std::cerr << "ERROR: No data for this element.  Use has_data() first!" << std::endl;
+      libMesh::err << "ERROR: No data for this element.  Use has_data() first!" << std::endl;
       libmesh_error();
     }
 #endif
@@ -1093,4 +1096,6 @@ void MeshData::set_unv_header (MeshDataUnvHeader* unv_header)
 // MeshDataUnvHeader inline methods
 
 
+} // namespace libMesh
+
 #endif
