@@ -1,4 +1,4 @@
-// $Id: mesh_refinement.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: mesh_refinement.h 4399 2011-04-22 18:32:39Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -35,13 +35,17 @@
 #include "libmesh_common.h"
 #include "libmesh.h" // libMesh::invalid_uint
 #include "location_maps.h"
+#include "elem.h"
+
+namespace libMesh
+{
 
 // Forward Declarations -----------------------------
 class MeshBase;
 class Point;
 class Node;
-class Elem;
 class ErrorVector;
+class PeriodicBoundaries;
 
 
 
@@ -73,6 +77,8 @@ private:
 
 public:
 
+  void set_periodic_boundaries_ptr(PeriodicBoundaries * pb_ptr);
+  
   /**
    * Destructor. Deletes all the elements that are currently stored.
    */
@@ -236,6 +242,12 @@ public:
    */
   void uniformly_p_coarsen (unsigned int n=1);
 
+  /**
+   * Sets the refinement flag to \p Elem::DO_NOTHING
+   * for each element in the mesh.
+   */
+  void clean_refinement_flags ();
+
   /**
    * Returns true if and only if the mesh is level one smooth
    * Returns false otherwise
@@ -558,12 +570,6 @@ private:
    * Updates the \p _new_nodes_map
    */
   void update_nodes_map ();
-
-  /**
-   * Sets the refinement flag to \p Elem::DO_NOTHING
-   * for each element in the mesh.
-   */
-  void clean_refinement_flags ();
   
   /**
    * Take user-specified coarsening flags and augment them
@@ -576,17 +582,32 @@ private:
    * so that level-one dependency is satisfied.
    */
   bool make_refinement_compatible (const bool);
-  
+
   /**
    * Copy refinement flags on ghost elements from their
    * local processors.  Return true if any flags changed.
    */
   bool make_flags_parallel_consistent ();
+
+  /**
+   * Local dispatch function for getting the correct topological
+   * neighbor from the Elem class
+   */
+  Elem* topological_neighbor (Elem* elem,
+                              const PointLocatorBase* point_locator,
+                              const unsigned int side);
+
+  /**
+   * Local dispatch function for checking the correct has_neighbor
+   * function from the Elem class
+   */
+  bool has_topological_neighbor (Elem* elem,
+                                 const PointLocatorBase* point_locator,
+                                 Elem* neighbor);
   
   /**
    * Data structure that holds the new nodes information.
    */
-  
   LocationMap<Node> _new_nodes_map;
 
   /**
@@ -621,6 +642,10 @@ private:
 
   unsigned char _face_level_mismatch_limit, _edge_level_mismatch_limit,
 	        _node_level_mismatch_limit;
+
+#ifdef LIBMESH_ENABLE_PERIODIC
+  PeriodicBoundaries * _periodic_boundaries;
+#endif
 };
 
 
@@ -685,5 +710,8 @@ inline unsigned char& MeshRefinement::node_level_mismatch_limit()
   return _node_level_mismatch_limit;
 }
 
+
+} // namespace libMesh
+
 #endif // end #ifdef LIBMESH_ENABLE_AMR
 #endif // end #ifndef __mesh_refinement_h__ 
