@@ -1,4 +1,4 @@
-// $Id: dense_matrix.h 3500 2009-10-19 18:28:00Z roystgnr $
+// $Id: dense_matrix.h 4233 2011-03-07 23:50:33Z jwpeterson $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -30,6 +30,9 @@
 #include "libmesh_common.h"
 #include "dense_matrix_base.h"
 
+namespace libMesh
+{
+
 // Forward Declarations
 template <typename T> class DenseVector;
 
@@ -107,12 +110,20 @@ public:
   virtual void right_multiply (const DenseMatrixBase<T>& M3);
 
   /**
-   * Perform matrix vector multiplication.
+   * Performs the matrix-vector multiplication,
+   * \p dest := (*this) * \p arg.
    */
   void vector_mult(DenseVector<T>& dest, const DenseVector<T>& arg) const;
+
+  /**
+   * Performs the matrix-vector multiplication,
+   * \p dest := (*this)^T * \p arg.
+   */
+  void vector_mult_transpose(DenseVector<T>& dest, const DenseVector<T>& arg) const;
   
   /**
-   * Perform matrix vector multiplication and add scaled result to \p dest.
+   * Performs the scaled matrix-vector multiplication,
+   * \p dest += \p factor * (*this) * \p arg. 
    */
   void vector_mult_add (DenseVector<T>& dest, 
                         const T factor,
@@ -267,11 +278,12 @@ public:
   { DenseMatrixBase<T>::condense (i, j, val, rhs); }
 
   /**
-   * Solve the system Ax=b given the input vector b.
+   * Solve the system Ax=b given the input vector b.  Partial pivoting
+   * is performed by default in order to keep the algorithm stable to
+   * the effects of round-off error.
    */
-  void lu_solve (DenseVector<T>& b,
-		 DenseVector<T>& x,
-		 const bool partial_pivot = false);
+  void lu_solve (const DenseVector<T>& b,
+		 DenseVector<T>& x);
 
 
 
@@ -288,6 +300,31 @@ public:
   void cholesky_solve(DenseVector<T2>& b,
 		      DenseVector<T2>& x);
 
+
+  /**
+   * Compute the Singular Value Decomposition of the matrix.
+   * On exit, sigma holds all of the singular values (in
+   * descending order).
+   *
+   * The implementation uses PETSc's interface to blas/lapack.
+   * If this is not available, this function throws an error.
+   */
+  void svd(DenseVector<T>& sigma);
+
+
+  /**
+   * Compute the "reduced" Singular Value Decomposition of the matrix.
+   * On exit, sigma holds all of the singular values (in
+   * descending order), U holds the left singular vectors,
+   * and VT holds the transpose of the right singular vectors.
+   * In the reduced SVD, U has min(m,n) columns and VT has
+   * min(m,n) rows. (In the "full" SVD, U and VT would be square.)
+   *
+   * The implementation uses PETSc's interface to blas/lapack.
+   * If this is not available, this function throws an error.
+   */
+  void svd(DenseVector<T>& sigma, DenseMatrix<T>& U, DenseMatrix<T>& VT);
+
   
   /**
    * @returns the determinant of the matrix.  Note that this means
@@ -311,7 +348,7 @@ public:
    * This was primarily used for testing purposes, and could be
    * removed...
    */
-  bool use_blas;
+  bool use_blas_lapack;
   
 private:
 
@@ -325,16 +362,15 @@ private:
    * is private since it is only called as part of the implementation
    * of the lu_solve(...) function.
    */
-  void _lu_decompose (const bool partial_pivot = false);
+  void _lu_decompose ();
   
   /**
    * Solves the system Ax=b through back substitution.  This function
    * is private since it is only called as part of the implementation
    * of the lu_solve(...) function.
    */
-  void _lu_back_substitute (DenseVector<T>& b,
-			    DenseVector<T>& x,
-			    const bool partial_pivot = false) const;
+  void _lu_back_substitute (const DenseVector<T>& b,
+			    DenseVector<T>& x) const;
   
   /**
    * Decomposes a symmetric positive definite matrix into a
@@ -361,7 +397,7 @@ private:
    * any desired outcome.  This typedef keeps track of which decomposition
    * has been called for this matrix.  
    */
-  enum DecompositionType {LU=0, CHOLESKY=1, NONE};
+  enum DecompositionType {LU=0, CHOLESKY=1, LU_BLAS_LAPACK, NONE};
 
   /**
    * This flag keeps track of which type of decomposition has been
@@ -370,9 +406,8 @@ private:
   DecompositionType _decomposition_type;
 
   /**
-   * Computes A <- op(A) * op(B) using BLAS gemm function.
-   * Used in the right_multiply(), left_multiply(), right_multiply_transpose(),
-   * and left_multiply_tranpose() routines.
+   * Enumeration used to determine the behavior of the _multiply_blas
+   * function.
    */
   enum _BLAS_Multiply_Flag {
     LEFT_MULTIPLY = 0,
@@ -381,8 +416,88 @@ private:
     RIGHT_MULTIPLY_TRANSPOSE
   };
   
+  /**
+   * The _multiply_blas function computes A <- op(A) * op(B) using
+   * BLAS gemm function.  Used in the right_multiply(),
+   * left_multiply(), right_multiply_transpose(), and
+   * left_multiply_tranpose() routines.
+   * [ Implementation in dense_matrix_blas_lapack.C ]
+   */
   void _multiply_blas(const DenseMatrixBase<T>& other,
 		      _BLAS_Multiply_Flag flag);
+
+  /**
+   * Computes an LU factorization of the matrix using the
+   * Lapack routine "getrf".  This routine should only be
+   * used by the "use_blas_lapack" branch of the lu_solve()
+   * function.  After the call to this function, the matrix
+   * is replaced by its factorized version, and the
+   * DecompositionType is set to LU_BLAS_LAPACK.
+   * [ Implementation in dense_matrix_blas_lapack.C ]
+   */
+  void _lu_decompose_lapack();
+
+  /**
+   * Computes an SVD of the matrix using the
+   * Lapack routine "getsvd".
+   * [ Implementation in dense_matrix_blas_lapack.C ]
+   */
+  void _svd_lapack(DenseVector<T>& sigma);
+  
+  /**
+   * Computes a "reduced" SVD of the matrix using the
+   * Lapack routine "getsvd".
+   * [ Implementation in dense_matrix_blas_lapack.C ]
+   */
+  void _svd_lapack(DenseVector<T>& sigma,
+                   DenseMatrix<T>& U,
+                   DenseMatrix<T>& VT);
+
+  /**
+   * Helper function that actually performs the SVD.
+   * [ Implementation in dense_matrix_blas_lapack.C ]
+   */
+  void _svd_helper (char JOBU,
+                    char JOBVT,
+                    std::vector<T>& sigma_val,
+                    std::vector<T>& U_val,
+                    std::vector<T>& VT_val);
+
+  /**
+   * This array is used to store pivot indices.  May be used
+   * by whatever factorization is currently active, clients of
+   * the class should not rely on it for any reason.
+   */
+  std::vector<int> _pivots;
+
+  /**
+   * Companion function to _lu_decompose_lapack().  Do not use
+   * directly, called through the public lu_solve() interface.
+   * This function is logically const in that it does not modify
+   * the matrix, but since we are just calling LAPACK routines,
+   * it's less const_cast hassle to just declare the function
+   * non-const.
+   * [ Implementation in dense_matrix_blas_lapack.C ]
+   */
+  void _lu_back_substitute_lapack (const DenseVector<T>& b,
+				   DenseVector<T>& x);
+
+  /**
+   * Uses the BLAS GEMV function (through PETSc) to compute
+   *
+   * dest := alpha*A*arg + beta*dest
+   *
+   * where alpha and beta are scalars, A is this matrix, and
+   * arg and dest are input vectors of appropriate size.  If
+   * trans is true, the transpose matvec is computed instead.
+   * By default, trans==false.
+   *
+   * [ Implementation in dense_matrix_blas_lapack.C ]
+   */
+  void _matvec_blas(T alpha, T beta,
+		    DenseVector<T>& dest,
+		    const DenseVector<T>& arg,
+		    bool trans=false) const;
 };
 
 
@@ -431,11 +546,13 @@ DenseMatrix<T>::DenseMatrix(const unsigned int m,
 			    const unsigned int n)
   : DenseMatrixBase<T>(m,n),
 #if defined(LIBMESH_HAVE_PETSC) && defined(LIBMESH_USE_REAL_NUMBERS)
-    use_blas(true),
+    use_blas_lapack(true),
 #else
-    use_blas(false),
+    use_blas_lapack(false),
 #endif
-    _decomposition_type(NONE)
+    _val(),
+    _decomposition_type(NONE),
+    _pivots()
 {
   this->resize(m,n);
 }
@@ -764,6 +881,8 @@ T DenseMatrix<T>::transpose (const unsigned int i,
 
 
 
+} // namespace libMesh
+
 
 
 
