@@ -1,4 +1,4 @@
-// $Id: distributed_vector.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: distributed_vector.h 4286 2011-03-24 03:55:50Z knezed01 $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -32,13 +32,15 @@
 // C++ includes
 #include <vector>
 #include <algorithm>
-#include <complex>
 #include <limits>
 
 // Local includes
 #include "numeric_vector.h"
 #include "parallel.h"
 
+namespace libMesh
+{
+
 
 
 /**
@@ -109,6 +111,13 @@ public:
    */
   void zero ();    
 
+  /**
+   * Creates a vector which has the same type, size and partitioning
+   * as this vector, but whose data is all zero.  Returns it in an \p
+   * AutoPtr.
+   */
+  virtual AutoPtr<NumericVector<T> > zero_clone () const;
+
   /**
    * Creates a copy of this vector and returns it in an \p AutoPtr.
    */
@@ -324,6 +333,16 @@ public:
    */
   void add_vector (const DenseVector<T>& V,
 		   const std::vector<unsigned int>& dof_indices);
+
+  /**
+   * \f$U+=A^T*V\f$.
+   * Add the product of the transpose of a Sparse matrix \p A_trans
+   * and a Numeric vector \p V to this Numeric vector.
+   * @e Not @e implemented.
+   */
+  void add_vector_transpose (const NumericVector<T>&,
+		             const SparseMatrix<T>&)
+  { libmesh_error(); }
   
   /**
    * \f$ U=v \f$ where v is a DenseVector<T> 
@@ -548,20 +567,15 @@ void DistributedVector<T>::init (const unsigned int n,
   
 #ifdef LIBMESH_HAVE_MPI
 
-  int n_proc=0, proc_id=0;
+  std::vector<int> local_sizes (libMesh::n_processors(), 0);
   
-  MPI_Comm_rank (libMesh::COMM_WORLD, &proc_id);
-  MPI_Comm_size (libMesh::COMM_WORLD, &n_proc);
-  
-  std::vector<int> local_sizes     (n_proc, 0);
-  
-  local_sizes[proc_id] = n_local;
+  local_sizes[libMesh::processor_id()] = n_local;
 
   Parallel::sum(local_sizes);
 
   // _first_local_index is the sum of _local_size
   // for all processor ids less than ours
-  for (int p=0; p<proc_id; p++)
+  for (unsigned int p=0; p!=libMesh::processor_id(); p++)
     _first_local_index += local_sizes[p];
 
 
@@ -570,7 +584,7 @@ void DistributedVector<T>::init (const unsigned int n,
   // size, otherwise there is big trouble!
   int sum=0;
 
-  for (int p=0; p<n_proc; p++)
+  for (unsigned int p=0; p!=libMesh::n_processors(); p++)
     sum += local_sizes[p];
 
   libmesh_assert (sum == static_cast<int>(n));
@@ -582,8 +596,8 @@ void DistributedVector<T>::init (const unsigned int n,
   // No other options without MPI!
   if (n != n_local)
     {
-      std::cerr << "ERROR:  MPI is required for n != n_local!"
-		<< std::endl;
+      libMesh::err << "ERROR:  MPI is required for n != n_local!"
+		    << std::endl;
       libmesh_error();
     }
   
@@ -679,6 +693,19 @@ void DistributedVector<T>::zero ()
 
 
 
+template <typename T>
+inline
+AutoPtr<NumericVector<T> > DistributedVector<T>::zero_clone () const
+{
+  AutoPtr<NumericVector<T> > cloned_vector (new DistributedVector<T>);
+
+  cloned_vector->init(*this);
+
+  return cloned_vector;
+}
+
+
+
 template <typename T>
 inline
 AutoPtr<NumericVector<T> > DistributedVector<T>::clone () const
@@ -851,5 +878,7 @@ void DistributedVector<T>::swap (NumericVector<T> &other)
   std::swap(_values, v._values);
 }
 
+} // namespace libMesh
+
 
 #endif  // #ifdef __distributed_vector_h__
