@@ -1,4 +1,4 @@
-// $Id: numeric_vector.h 3456 2009-09-04 18:32:19Z roystgnr $
+// $Id: numeric_vector.h 4286 2011-03-24 03:55:50Z knezed01 $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -35,6 +35,9 @@
 #include "reference_counted_object.h"
 #include "libmesh.h"
 
+namespace libMesh
+{
+
 
 // forward declarations
 template <typename T> class NumericVector;
@@ -140,6 +143,14 @@ public:
    */
   virtual void zero () = 0;    
 
+  /**
+   * Creates a vector which has the same type, size and partitioning
+   * as this vector, but whose data is all zero.  Returns it in an \p
+   * AutoPtr.
+   * This must be overloaded in the derived classes.
+   */
+  virtual AutoPtr<NumericVector<T> > zero_clone () const = 0;
+  
   /**
    * Creates a copy of this vector and returns it in an \p AutoPtr.
    * This must be overloaded in the derived classes.
@@ -250,7 +261,7 @@ public:
    * Note that the indices must necessary live on this
    * processor.
    */
-  virtual Real subset_l1_norm (const std::set<unsigned int> & indices);
+  virtual Real subset_l1_norm (const std::set<unsigned int> & indices) const;
 
   /**
    * @returns the \f$l_2\f$-norm of the vector, i.e.
@@ -261,7 +272,7 @@ public:
    * Note that the indices must necessary live on this
    * processor.
    */
-  virtual Real subset_l2_norm (const std::set<unsigned int> & indices);
+  virtual Real subset_l2_norm (const std::set<unsigned int> & indices) const;
 
   /**
    * @returns the maximum absolute value of the
@@ -271,7 +282,7 @@ public:
    * Note that the indices must necessary live on this
    * processor.
    */
-  virtual Real subset_linfty_norm (const std::set<unsigned int> & indices);
+  virtual Real subset_linfty_norm (const std::set<unsigned int> & indices) const;
 
   /**
    * @returns dimension of the vector. This
@@ -416,6 +427,13 @@ public:
   virtual void add_vector (const DenseVector<T>& V,
 			   const std::vector<unsigned int>& dof_indices) = 0;
 
+  /**
+   * \f$U+=A^T*V\f$, add the product of the transpose of a \p SparseMatrix \p A_trans
+   * and a \p NumericVector \p V to \p this, where \p this=U.
+   */
+  virtual void add_vector_transpose (const NumericVector<T>&,
+			             const SparseMatrix<T>&) = 0;
+
   /**
    * \f$ U=v \f$ where v is a std::vector<T> 
    * and you want to specify WHERE to insert it
@@ -506,14 +524,36 @@ public:
   /**
    * @returns \p -1 when \p this is equivalent to \p other_vector,
    * up to the given \p threshold.  When differences occur,
-   * the return value contains the first index where
-   * the difference exceeded the threshold.  When
+   * the return value contains the first index \p i where
+   * the difference \p (a[i]-b[i]) exceeded the threshold.  When
    * no threshold is given, the \p libMesh \p TOLERANCE
    * is used.
    */
   virtual int compare (const NumericVector<T> &other_vector,
 		       const Real threshold = TOLERANCE) const;
 
+  /**
+   * @returns \p -1 when \p this is equivalent to \p other_vector,
+   * up to the given local relative \p threshold.  When differences
+   * occur, the return value contains the first index where
+   * the difference \p (a[i]-b[i])/max(a[i],b[i]) exceeded the
+   * threshold.  When no threshold is given, the \p libMesh 
+   * \p TOLERANCE is used.
+   */
+  virtual int local_relative_compare (const NumericVector<T> &other_vector,
+                                      const Real threshold = TOLERANCE) const;
+
+  /**
+   * @returns \p -1 when \p this is equivalent to \p other_vector,
+   * up to the given local relative \p threshold.  When differences
+   * occur, the return value contains the first index where
+   * the difference \p (a[i]-b[i])/max_j(a[j],b[j]) exceeded the
+   * threshold.  When no threshold is given, the \p libMesh 
+   * \p TOLERANCE is used.
+   */
+  virtual int global_relative_compare (const NumericVector<T> &other_vector,
+                                       const Real threshold = TOLERANCE) const;
+
   /**
    * Computes the pointwise (i.e. component-wise) product of \p vec1
    * and \p vec2 and stores the result in \p *this.
@@ -522,14 +562,16 @@ public:
 			       const NumericVector<T>& vec2) = 0;
 
   /**
-   * Prints the local contents of the vector to the screen.
+   * Prints the local contents of the vector, by default to
+   * libMesh::out
    */
-  virtual void print(std::ostream& os=std::cout) const;
+  virtual void print(std::ostream& os=libMesh::out) const;
 
   /**
-   * Prints the global contents of the vector to the screen.
+   * Prints the global contents of the vector, by default to
+   * libMesh::out
    */
-  virtual void print_global(std::ostream& os=std::cout) const;
+  virtual void print_global(std::ostream& os=libMesh::out) const;
 
   /**
    * Same as above but allows you to use stream syntax.
@@ -548,8 +590,8 @@ public:
    */
   virtual void print_matlab(const std::string name="NULL") const
   {
-    std::cerr << "ERROR: Not Implemented in base class yet!" << std::endl;
-    std::cerr << "ERROR writing MATLAB file " << name << std::endl;
+    libMesh::err << "ERROR: Not Implemented in base class yet!" << std::endl;
+    libMesh::err << "ERROR writing MATLAB file " << name << std::endl;
     libmesh_error();
   }
 
@@ -562,7 +604,7 @@ public:
   virtual void create_subvector(NumericVector<T>& ,
 				const std::vector<unsigned int>& ) const
   {
-    std::cerr << "ERROR: Not Implemented in base class yet!" << std::endl;
+    libMesh::err << "ERROR: Not Implemented in base class yet!" << std::endl;
     libmesh_error();
   }
     
@@ -812,5 +854,7 @@ void  NumericVector<T>::swap (NumericVector<T> &v)
 }
 
 
+} // namespace libMesh
+
 
 #endif  // #ifdef __numeric_vector_h__
