@@ -1,4 +1,4 @@
-// $Id: petsc_vector.h 3459 2009-09-08 16:10:30Z roystgnr $
+// $Id: petsc_vector.h 4286 2011-03-24 03:55:50Z knezed01 $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -31,6 +31,7 @@
 
 // C++ includes
 #include <vector>
+#include "string.h" // Needed for strcmp with gcc-4.4.3
 
 // Local includes
 #include "numeric_vector.h"
@@ -43,6 +44,9 @@ EXTERN_C_FOR_PETSC_BEGIN
 # include <petscvec.h>
 EXTERN_C_FOR_PETSC_END
 
+namespace libMesh
+{
+
 
 
 // forward declarations
@@ -122,6 +126,13 @@ public:
    */
   void zero ();    
 
+  /**
+   * Creates a vector which has the same type, size and partitioning
+   * as this vector, but whose data is all zero.  Returns it in an \p
+   * AutoPtr.
+   */
+  virtual AutoPtr<NumericVector<T> > zero_clone () const;
+  
   /**
    * Creates a copy of this vector and returns it in an \p AutoPtr.
    */
@@ -363,6 +374,14 @@ public:
    */
   void add_vector (const DenseVector<T>& V,
 		   const std::vector<unsigned int>& dof_indices);
+
+  /**
+   * \f$U+=A^T*V\f$, add the product of the transpose
+   * of \p SparseMatrix \p A_trans and a \p NumericVector \p V to
+   * \p this, where \p this=U.
+   */
+  void add_vector_transpose (const NumericVector<T> &V,
+		             const SparseMatrix<T> &A_trans);
   
   /**
    * \f$ U=v \f$ where v is a std::vector<T> 
@@ -638,35 +657,41 @@ PetscVector<T>::PetscVector (Vec v)
 
   /* We need to ask PETSc about the (local to global) ghost value
      mapping and create the inverse mapping out of it.  */
-  int ierr=0, petsc_size=0, petsc_local_size=0;
-  ierr = VecGetSize(_vec, &petsc_size);
-  CHKERRABORT(libMesh::COMM_WORLD,ierr);
+  int ierr=0;
+  int petsc_local_size=0;
   ierr = VecGetLocalSize(_vec, &petsc_local_size);
   CHKERRABORT(libMesh::COMM_WORLD,ierr);
 
-  /* \p petsc_local_size is the number of non-ghost values.  If it
-     equals the global size, then we are a serial vector, and there
-     are no ghost values.  */
-  if(petsc_size!=petsc_local_size)
-    {
-      ISLocalToGlobalMapping mapping = _vec->mapping;
+  // Get the vector type from PETSc.
+  // As of Petsc 3.0.0, the VecType #define lost its const-ness, so we
+  // need to have it in the code
+#if PETSC_VERSION_LESS_THAN(3,0,0)
+  VecType type;
+#else
+  const VecType type;
+#endif
+  ierr = VecGetType(_vec, &type);
+  CHKERRABORT(libMesh::COMM_WORLD,ierr);
 
-      // If is a sparsely stored vector, set up our new mapping
-      if (mapping)
-        {
-          const unsigned int local_size = static_cast<unsigned int>(petsc_local_size);
-          const unsigned int ghost_begin = static_cast<unsigned int>(petsc_local_size);
-          const unsigned int ghost_end = static_cast<unsigned int>(mapping->n);
-          for(unsigned int i=ghost_begin; i<ghost_end; i++)
-	    _global_to_local_map[mapping->indices[i]] = i-local_size;
-	  this->_type = GHOSTED;
-	}
-      else
-	this->_type = PARALLEL;
+  if((strcmp(type,VECSHARED) == 0) || (strcmp(type,VECMPI) == 0))
+  {
+    ISLocalToGlobalMapping mapping = _vec->mapping;
+
+    // If is a sparsely stored vector, set up our new mapping
+    if (mapping)
+    {
+      const unsigned int local_size = static_cast<unsigned int>(petsc_local_size);
+      const unsigned int ghost_begin = static_cast<unsigned int>(petsc_local_size);
+      const unsigned int ghost_end = static_cast<unsigned int>(mapping->n);
+      for(unsigned int i=ghost_begin; i<ghost_end; i++)
+        _global_to_local_map[mapping->indices[i]] = i-local_size;
+      this->_type = GHOSTED;
     }
+    else
+      this->_type = PARALLEL;
+  }
   else
     this->_type = SERIAL;
-
 }
 
 
@@ -722,11 +747,16 @@ void PetscVector<T>::init (const unsigned int n,
   // otherwise create an MPI-enabled vector
   else if (this->_type == PARALLEL)
     {
+#ifdef LIBMESH_HAVE_MPI
       libmesh_assert (n_local <= n);
-      
       ierr = VecCreateMPI (libMesh::COMM_WORLD, petsc_n_local, petsc_n,
 			   &_vec);
              CHKERRABORT(libMesh::COMM_WORLD,ierr);
+#else
+      libmesh_assert (n_local == n);
+      ierr = VecCreateSeq (PETSC_COMM_SELF, petsc_n, &_vec);
+             CHKERRABORT(PETSC_COMM_SELF,ierr);
+#endif
       
       ierr = VecSetFromOptions (_vec);
              CHKERRABORT(libMesh::COMM_WORLD,ierr);
@@ -735,7 +765,7 @@ void PetscVector<T>::init (const unsigned int n,
     libmesh_error();
   
   this->_is_initialized = true;
-//  this->_is_closed = true;
+  this->_is_closed = true;
   
   
   if (fast == false)
@@ -768,11 +798,16 @@ void PetscVector<T>::init (const unsigned int n,
   int petsc_n_local=static_cast<int>(n_local);
   int petsc_n_ghost=static_cast<int>(ghost.size());
 
-  // If the mesh is disjoint, the following assertion will fail.
   // If the mesh is not disjoint, every processor will either have
   // all the dofs, none of the dofs, or some non-zero dofs at the
   // boundary between processors.
-  libmesh_assert(n_local == 0 || n_local == n || !ghost.empty());
+  //
+  // However we can't assert this, because someone might want to
+  // construct a GHOSTED vector which doesn't include neighbor element
+  // dofs.  Boyce tried to do so in user code, and we're going to want
+  // to do so in System::project_vector().
+  //
+  // libmesh_assert(n_local == 0 || n_local == n || !ghost.empty());
 
   int* petsc_ghost = ghost.empty() ? PETSC_NULL :
     const_cast<int*>(reinterpret_cast<const int*>(&ghost[0]));
@@ -800,7 +835,7 @@ void PetscVector<T>::init (const unsigned int n,
   CHKERRABORT(libMesh::COMM_WORLD,ierr);
   
   this->_is_initialized = true;
-//  this->_is_closed = true;
+  this->_is_closed = true;
   
   if (fast == false)
     this->zero ();
@@ -937,6 +972,19 @@ void PetscVector<T>::zero ()
 
 
 
+template <typename T>
+inline
+AutoPtr<NumericVector<T> > PetscVector<T>::zero_clone () const
+{
+  AutoPtr<NumericVector<T> > cloned_vector (new PetscVector<T>);
+
+  cloned_vector->init(*this);
+
+  return cloned_vector;
+}
+
+
+
 template <typename T>
 inline
 AutoPtr<NumericVector<T> > PetscVector<T>::clone () const
@@ -1204,6 +1252,8 @@ void PetscVector<T>::_restore_array(void) const
 }
 
 
+} // namespace libMesh
+
 
 #endif // #ifdef LIBMESH_HAVE_PETSC
 #endif // #ifdef __petsc_vector_h__
