@@ -1,4 +1,4 @@
-// $Id: trilinos_epetra_vector.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: trilinos_epetra_vector.h 4286 2011-03-24 03:55:50Z knezed01 $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -41,6 +41,9 @@
 class Epetra_IntSerialDenseVector;
 class Epetra_SerialDenseVector;
 
+namespace libMesh
+{
+
 // forward declarations
 template <typename T> class SparseMatrix;
 
@@ -118,6 +121,13 @@ public:
    */
   void zero ();    
 
+  /**
+   * Creates a vector which has the same type, size and partitioning
+   * as this vector, but whose data is all zero.  Returns it in an \p
+   * AutoPtr.
+   */
+  virtual AutoPtr<NumericVector<T> > zero_clone () const;
+
   /**
    * Creates a copy of this vector and returns it in an \p AutoPtr.
    */
@@ -344,6 +354,13 @@ public:
    */
   void add_vector (const DenseVector<T>& V,
 		   const std::vector<unsigned int>& dof_indices);
+
+  /**
+   * \f$U+=A*V\f$, add the product of the transpose of a \p SparseMatrix \p A_trans
+   * and a \p NumericVector \p V to \p this, where \p this=U.
+   */
+  void add_vector_transpose (const NumericVector<T> &V,
+		             const SparseMatrix<T> &A_trans);
   
   /**
    * \f$ U=v \f$ where v is a DenseVector<T> 
@@ -729,6 +746,9 @@ void EpetraVector<T>::init (const unsigned int n,
 			    const bool fast,
                             const ParallelType type)
 {
+  // We default to allocating n_local local storage
+  unsigned int my_n_local = n_local;
+
   if (type == AUTOMATIC)
     {
       if (n == n_local)
@@ -736,14 +756,22 @@ void EpetraVector<T>::init (const unsigned int n,
       else
         this->_type = PARALLEL;
     }
+  else if (type == GHOSTED)
+    {
+      // We don't yet support GHOSTED Epetra vectors, so to get the
+      // same functionality we need a SERIAL vector with local
+      // storage allocated for every entry.
+      this->_type = SERIAL;
+      my_n_local = n;
+    }
   else
     this->_type = type;
 
-  libmesh_assert ((this->_type==SERIAL && n==n_local) ||
+  libmesh_assert ((this->_type==SERIAL && n==my_n_local) ||
                   this->_type==PARALLEL);
 
   _map = new Epetra_Map(n, 
-                        n_local, 
+                        my_n_local, 
                         0, 
                         Epetra_MpiComm (libMesh::COMM_WORLD));
 	      
@@ -825,6 +853,19 @@ void EpetraVector<T>::zero ()
 
 
 
+template <typename T>
+inline
+AutoPtr<NumericVector<T> > EpetraVector<T>::zero_clone () const
+{
+  AutoPtr<NumericVector<T> > cloned_vector (new EpetraVector<T>);
+
+  cloned_vector->init(*this);
+
+  return cloned_vector;
+}
+
+
+
 template <typename T>
 inline
 AutoPtr<NumericVector<T> > EpetraVector<T>::clone () const
@@ -934,6 +975,8 @@ void EpetraVector<T>::swap (NumericVector<T> &other)
   std::swap(_destroy_vec_on_exit, v._destroy_vec_on_exit);
 }
 
+} // namespace libMesh
+
 
 #endif // #ifdef HAVE_EPETRA
 #endif // #ifdef __trilinos_epetra_vector_h__
