@@ -1,4 +1,4 @@
-// $Id: type_tensor.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: type_tensor.h 3874 2010-07-02 21:57:26Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -29,6 +29,9 @@
 #include "libmesh_common.h"
 #include "type_vector.h"
 
+namespace libMesh
+{
+
 
 
 
@@ -109,6 +112,11 @@ public:
    */
   T & operator () (const unsigned int i, const unsigned int j);
 
+  /**
+   * Return one row of the tensor as a TypeVector.
+   */
+  TypeVector<T> row(const unsigned int r);
+
   /**
    * Add two tensors. 
    */
@@ -270,9 +278,9 @@ public:
   bool operator > (const TypeTensor<T>& rhs) const;
   
   /**
-   * Formatted print to \p std::cout.
+   * Formatted print, by default to \p libMesh::out.
    */
-  void print(std::ostream& os) const;
+  void print(std::ostream& os = libMesh::out) const;
 
   /**
    * Formatted print as above but allows you to do
@@ -433,7 +441,7 @@ T & TypeTensor<T>::operator () (const unsigned int i,
 
   if (i >= LIBMESH_DIM || j >= LIBMESH_DIM)
     {
-//       std::cerr << "ERROR:  You are assigning to a tensor component" << std::endl
+//       libMesh::err << "ERROR:  You are assigning to a tensor component" << std::endl
 // 		<< "that is out of range for the compiled LIBMESH_DIM!"      << std::endl
 // 		<< " LIBMESH_DIM=" << LIBMESH_DIM << " , i=" << i << " , j=" << j << std::endl;
       libmesh_error();
@@ -447,7 +455,17 @@ T & TypeTensor<T>::operator () (const unsigned int i,
   return _coords[i*LIBMESH_DIM+j];
 }
 
+template <typename T>
+TypeVector<T>
+TypeTensor<T>::row(const unsigned int r)
+{
+  TypeVector<T> return_vector;
 
+  for(unsigned int j=0; j<LIBMESH_DIM; j++)
+    return_vector._coords[j] = _coords[r*LIBMESH_DIM + j];
+
+  return return_vector;
+}
 
 template <typename T>
 template<typename T2>
@@ -921,4 +939,6 @@ bool TypeTensor<T>::operator == (const TypeTensor<T>& rhs) const
   
 }
 
+} // namespace libMesh
+
 #endif // #define __type_tensor_h__
