@@ -1,4 +1,4 @@
-// $Id: type_vector.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: type_vector.h 4389 2011-04-21 15:57:46Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -29,6 +29,9 @@
 #include "libmesh_common.h"
 #include "compare_types.h"
 
+namespace libMesh
+{
+
 // Forward declaration for friend class
 template <typename T>
 class TypeTensor;
@@ -256,9 +259,9 @@ public:
   bool operator > (const TypeVector<T>& rhs) const;
 
   /**
-   * Formatted print to \p std::cout.
+   * Formatted print, by default to \p libMesh::out.
    */
-  void print(std::ostream& os) const;
+  void print(std::ostream& os = libMesh::out) const;
 
   /**
    * Formatted print as above but allows you to do
@@ -304,7 +307,11 @@ TypeVector<T>::TypeVector (const T x,
 
       if (LIBMESH_DIM == 3)
 	_coords[2] = z;
+      else
+        libmesh_assert(z == 0.);
     }
+  else
+    libmesh_assert(y == 0.);
 }
 
 
@@ -344,15 +351,8 @@ template <typename T>
 inline
 T TypeVector<T>::operator () (const unsigned int i) const
 {
-  libmesh_assert (i<3);
+  libmesh_assert (i<LIBMESH_DIM);
 
-#if LIBMESH_DIM < 3
-  
-  if (i > (LIBMESH_DIM-1))
-    return 0.;
-  
-#endif
-  
   return _coords[i];
 }
 
@@ -362,19 +362,6 @@ template <typename T>
 inline
 T & TypeVector<T>::operator () (const unsigned int i)
 {
-#if LIBMESH_DIM < 3
-
-  if (i >= LIBMESH_DIM)
-    {
-//       std::cerr << "ERROR:  You are assigning to a vector component" << std::endl
-// 		<< "that is out of range for the compiled LIBMESH_DIM!"      << std::endl
-// 		<< " LIBMESH_DIM=" << LIBMESH_DIM << " , i=" << i
-// 		<< std::endl;
-      libmesh_error();
-    }
-  
-#endif
-  
   libmesh_assert (i<LIBMESH_DIM);
   
   return _coords[i];
@@ -829,4 +816,6 @@ bool TypeVector<Real>::operator != (const TypeVector<Real>& rhs) const
   return (!(*this == rhs));
 }
 
+} // namespace libMesh
+
 #endif // #define __type_vector_h__
