@@ -1,4 +1,4 @@
-// $Id: number_lookups.h 3391 2009-05-26 03:50:35Z benkirk $
+// $Id: number_lookups.h 3874 2010-07-02 21:57:26Z roystgnr $
 
 // The libMesh Finite Element Library.
 // Copyright (C) 2002-2008 Benjamin S. Kirk, John W. Peterson, Roy H. Stogner
@@ -21,6 +21,9 @@
 #ifndef __number_lookups_h__
 #define __number_lookups_h__
 
+namespace libMesh
+{
+
 // Lookup tables for hierarchic numbering of basis functions
 extern const unsigned char triangular_number_row[];
 extern const unsigned char triangular_number_column[];
@@ -30,4 +33,6 @@ extern const unsigned char cube_number_row[];
 extern const unsigned char cube_number_column[];
 extern const unsigned char cube_number_page[];
 
+} // namespace libMesh
+
 #endif // ifndef __number_lookups_h__
